// #ifdef H5
import wx from 'weixin-js-sdk'
// #endif
import {
  isWeixin
} from "./common.js"
import store from '@/store/index.js'

import md5 from "md5"

export function webViewHref(url, pam = {}) {

  var defaultPam = {
    ...pam,
    "x-token": store.state.user.token,
    openId: store.state.user.openId,
    userId: store.state.user.userId,
    username: store.state.user.name,
    useravatar: store.state.user.avatar,
    // #ifndef H5
    miniProgram: "jsm_env",
    // #endif
  }

  var pamstr = Object.entries(defaultPam).map(item => item.join("=")).join("&");

  if (!!~url.indexOf("?")) {
    return process.env.VUE_APP_HREF + url + `${pamstr?"&"+pamstr:""}`
  } else {
    return process.env.VUE_APP_HREF + url + `${pamstr?"?"+pamstr:""}`
  }

}

export const navToPage = (function() {
  var gopageid = null
  return function(p, goToType = "navigateTo") {
    var {
      url,
      ...data
    } = p
    // #ifdef H5
    if (isWeixin()) {
      wx.miniProgram.getEnv(res => {
        if (res.miniprogram) {
          var i = 0;
          (function goTo() {
            if (gopageid) {
              clearTimeout(gopageid)
            }
            i++
            if (uniWebview && uniWebview[goToType]) {
              // h5跳转小程序
              if (uni.getStorageSync('miniProgram') == "jsm_env") {
                // 家盛茂小程序环境
                uniWebview[goToType]({
                  url: url,
                  ...data
                })
              } else {
                // 原生小程序页面
                var miniProgramPages = []
                if (!!~miniProgramPages.indexOf(url.split("?")[0])) {
                  // 第三方小程序跳转我们的h5页面处理
                  uniWebview[goToType]({
                    url: `/pagesJsm/pages/middle?webUrl=${encodeURIComponent(webViewHref(url))}`,
                    ...data
                  })
                } else {
                  // 第三方小程序跳转我们的h5页面处理
                  uniWebview[goToType]({
                    url: `/pagesJsm/pages/webview?webUrl=${encodeURIComponent(webViewHref(url))}`,
                    ...data
                  })
                }
              }
            } else if (i < 10) {
              gopageid = setTimeout(function() {
                gopageid = null
                goTo()
              }, 200)
            }
          })()
        } else {
          // 当前环境跳转
          uni[goToType]({
            url: url,
            ...data
          })
        }
      })
    } else {
      // 当前环境跳转
      uni[goToType]({
        url: url,
        ...data
      })
    }
    // #endif
    // #ifndef H5
    // 当前环境跳转
    uni[goToType]({
      url: url,
      ...data
    })
    // #endif
  }
})()

export const crossPage = (function() {
  var funs = {}
  var tiemIdIs = {}
  var funco = {
    $emit(name, data) {
      if (funs[name]) {
        Object.keys(funs[name]).map(key => {
          funs[name][key](data)
        })
      }
      // #ifndef H5
      
      // #endif
      
      // #ifdef H5
      var postMessage = (window.uniWebview && window.uniWebview.postMessage) || (window.postMessage)
      if (postMessage) {
        postMessage({
          data: {
            type: "crossPage_ts",
            name,
            data
          }
        })
      }
      if (tiemIdIs[name]) {
        clearTimeout(tiemIdIs[name])
      }
      uni.setStorageSync(name, data);
      tiemIdIs[name] = setTimeout(function() {
        uni.removeStorageSync(name)
      }, 350)
      // #endif
    },

    $on(name, fun) {
      if (!funs[name]) {
        funs[name] = {}
      }
      funs[name][md5(fun.toString())] = fun
    },

    $off(name, fun) {
      var skey = md5(fun.toString())
      if (funs[name] && funs[name][skey]) {
        delete funs[name][skey]
      }
    },

    $listener(e) {
      e.detail.data.map(({
        type,
        name,
        data
      }) => {
        if (type == "crossPage_ts") {
          if (funs[name]) {
            Object.keys(funs[name]).map(key => {
              funs[name][key](data)
            })
          }
        }
      })
    },
  };
  // #ifdef H5
  (function tiemID() {
    Object.keys(funs).map(name => {
      if (funs[name]) {
        var data = uni.getStorageSync(name);
        if (data) {
          Object.keys(funs[name]).map(key => {
            funs[name][key](data)
          })
        }
      }
    })
    setTimeout(tiemID, 200)
  })()
  // #endif
  return funco
})()